using System.Collections;
using System.Collections.Generic;
using System;
using UnityEngine;
using TMPro;
using UnityEngine.UI;

public class AImodule : IComparable<AImodule>
{

    public string Name = "Common"; //used to identify the tile, this is the default
    public NHC HostClass;
    public TextMeshPro NameText;

    public float LoopTimeGoal;
    public float LoopAmt = 0.5f;
    public float MoveChangeTimer;
    public float MoveChangeAmt = UnityEngine.Random.Range(0.5f, 7f);
    public float TalkStateTimer;
    public float TalkStateAmt = UnityEngine.Random.Range(1f, 6f);
    public float TalkChangeTimer;
    public float TalkChangeAmt = UnityEngine.Random.Range(0.5f, 3f);

    public GameObject NameBubble = null;
    public GameObject TalkingBubble = null;

    public int WalkDirection = RNG11();
    public bool Walking = false;
    public bool Talking = false;

    public AImodule(string Name_, NHC Class) //AI module start / creation
    {
        HostClass = Class;
        Name = Name_;

        float boundsY;
        switch(Name) 
        {
        case "Walking_Random": //majorly using update, walks around randomly
            Walking = true; //inverts in first tick, ignore this.
            LoopAmt = 0;
            HostClass.ChangeDir(WalkDirection);
            break;
        case "Falling": //allows them to be in air and override states
            LoopAmt = 0.1f;
            break;
        case "Attack_Gun_Random": //attacks randomly, majorly using update
            LoopAmt = UnityEngine.Random.Range(0.5f, 10);
            HostClass.AIchancePool ++; //adds into the chance pool!
            break;
        case "NameTags": //a cool nametag thing above the head area
            LoopAmt = 0.2f;
            NameBubble = new GameObject("NameTag");
            NameBubble.name = ("NameTag-" + UnityEngine.Random.Range(10000000,99999999));
            NameText = NameBubble.AddComponent<TextMeshPro>();
            NameText.alignment = TextAlignmentOptions.Center;
            NameBubble.transform.SetParent(HostClass.gameObject.transform, true);
            boundsY = ((HostClass.gameObject.GetComponent<Collider2D>().bounds.size.y / HostClass.gameObject.transform.localScale.y));
            NameBubble.transform.localPosition = new Vector3(0,boundsY - 1.3f);
            //NameBubble.transform.localScale = HostClass.gameObject.transform.localScale;
            NameText.fontSize = 1f;
            if (HostClass.Name == null) HostClass.Name = HostClass.PossibleNames[UnityEngine.Random.Range(0, (HostClass.PossibleNames.Length) - 1)]; //sets the name to a random name if null
            NameText.text = HostClass.Name;
            break;
        case "Names_Common": //fun fact, order matters when adding AI modules, do this before Name Tags plz, or etc... idk..
            HostClass.PossibleNames = AssetLoadup.CommonNames();
            break;
        case "Damage_FromEnvo": //can take damage from the world
            HostClass.TakeDamageFromEnvo = true;
            break;
        case "Damage_FromShot": //can take damage from being shot
            HostClass.TakeDamageFromShot = true;
            break;
        case "Damage_FriendlyFire": //can take damage from other NHCs
            HostClass.FriendlyFire = true;
            break;
        case "Talking": //talks randomly and has basic interactions
            LoopAmt = 0.2f;
            HostClass.AIchancePool ++; //adds into the chance pool!
            TalkingBubble = new GameObject("TalkingBubble");
            TalkingBubble.name = ("TalkingBubble-" + UnityEngine.Random.Range(10000000,99999999));
            SpriteRenderer BubbleSprite = TalkingBubble.AddComponent<SpriteRenderer>();
            BubbleSprite.sprite = AssetLoadup.Talk_Bubbles[UnityEngine.Random.Range(0, (AssetLoadup.Talk_Bubbles.Length - 1))];
            BubbleSprite.enabled = false;

            Talking = true; //inverts in first tick, ignore this.
            boundsY = ((HostClass.gameObject.GetComponent<Collider2D>().bounds.size.y / HostClass.gameObject.transform.localScale.y));
            TalkingBubble.transform.localPosition = new Vector3(0,boundsY - 1f);
            TalkingBubble.transform.localScale = HostClass.gameObject.transform.localScale;

            TalkingBubble.transform.SetParent(HostClass.gameObject.transform, true);
            break;
        case "Jumping":
            HostClass.CanJump = true;
            break;
        case "JumpGaps":
            LoopAmt = 0.05f;
            HostClass.JumpGaps = true;
            break;
        case "Slap_Meme": //slaps things randomly
            LoopAmt = 0.8f;
            break;
        default:
            ModAPI.Notify("<color=#ff35005>NHC AI module " + Name + " does not exist!</color>");
            break;
        }
        LoopTimeGoal = (UnityEngine.Time.fixedTime + LoopAmt); //resets the loop timer so it doesn't instantly activate everything
    }

    public void AImoduleTick(){
        if (UnityEngine.Time.fixedTime > LoopTimeGoal){
            LoopTimeGoal = (UnityEngine.Time.fixedTime + LoopAmt);

            AIfunc(Name);
        }
    }

    public bool CheckMoveDir(int Dir){
        Vector2 Pos = HostClass.gameObject.transform.position + new Vector3(0, -0.7f);

        RaycastHit2D WalkCheckSurface;
        bool tempbool = false;
        tempbool = ((Physics2D.Linecast(
            Pos,
            Pos + new Vector2((Dir * HostClass.MoveCheckRange),0),
            1 << LayerMask.NameToLayer("Objects"))
            ) ||
            (Physics2D.Linecast(
            Pos,
            Pos + new Vector2((Dir * HostClass.MoveCheckRange),0),
            1 << LayerMask.NameToLayer("Bounds")))
            );

        return(tempbool);
    }

    public static int RNG11(){ //returns either -1 or 1
        int temp = (int) UnityEngine.Random.Range(0,2)*2-1;
        //UnityEngine.Debug.Log("RNG " + temp);
        return(temp);
    }

    public void AIfunc(string Func){ //runs per tick of the AI module
        float boundsY;
        switch(Func) 
        {
        case "Walking_Random": //walks around randomly but is also just general walking.
            LoopAmt = 0;
            if (HostClass.InAir || HostClass.Dead){ //stops walking constantly if in air
                Walking = false;
            }

            if (!(HostClass.InAir || HostClass.Dead)){
                if (HostClass.PlayerControlling){ //if player is controlling, then allow control, otherwise do normal code.
                    if (Walking){
                        if (!(InputSystem.Held(AssetLoadup.KeyRight) || InputSystem.Held(AssetLoadup.KeyLeft))){
                        UnityEngine.Debug.Log("NHC Controller Stopped Walking.");
                        HostClass.SetState("Stand");
                        Walking = false;
                    }
                    }else if (InputSystem.Held(AssetLoadup.KeyRight) || InputSystem.Held(AssetLoadup.KeyLeft)){
                            if (InputSystem.Held(AssetLoadup.KeyLeft)){
                                UnityEngine.Debug.Log("NHC Controller Started Walking LEFT");
                                WalkDirection = -1;
                            }
                            if (InputSystem.Held(AssetLoadup.KeyRight)){
                                UnityEngine.Debug.Log("NHC Controller Started Walking RIGHT");
                                WalkDirection = 1;
                            }
                            HostClass.ChangeDir(WalkDirection);
                            HostClass.SetState("Walk");
                            Walking = true;
                        }
                }else{ //normal non-player control.
                    if (UnityEngine.Time.fixedTime > MoveChangeTimer){ //direction changing and stop-walking code
                        MoveChangeTimer = (UnityEngine.Time.fixedTime + MoveChangeAmt);
                        MoveChangeAmt = UnityEngine.Random.Range(0.5f, 7f);

                        Walking = !Walking;
                        if (Walking){
                            WalkDirection = RNG11();
                            if (!CheckMoveDir(WalkDirection)){
                                HostClass.SetState("Walk");
                            }else{
                                if (!CheckMoveDir(WalkDirection * -1)){
                                    WalkDirection *= -1;
                                    HostClass.SetState("Walk");
                                }else{
                                    Walking = false;
                                }
                            }
                        }else{
                            HostClass.SetState("Stand");
                        }
                        HostClass.ChangeDir(WalkDirection);
                    }
                }
            }
            if (Walking){ //constant walking code
                if (!CheckMoveDir(WalkDirection)){
                    if (!(HostClass.RigBody.bodyType == RigidbodyType2D.Static)){ //set velocity if not static.
                    HostClass.RigBody.velocity = new Vector2(HostClass.MoveSpeed * WalkDirection, HostClass.RigBody.velocity.y);
                    }
                }else{
                    if (Walking){
                        HostClass.SetState("Stand");
                        Walking = false;
                    }
                }
            }
            break;
        case "Attack_Gun_Random":
            LoopAmt = UnityEngine.Random.Range(0.5f, 10);
            if (!HostClass.Dead && AIchancePoolCheck()){
                if (HostClass.AniState == "Stand"){
                    HostClass.SetState("Gun");
                }
            }
            break;
        case "NameTags":
            NameBubble.transform.localScale = HostClass.gameObject.transform.localScale;
            boundsY = ((HostClass.gameObject.GetComponent<Collider2D>().bounds.size.y / HostClass.gameObject.transform.localScale.y));
            NameBubble.transform.localPosition = new Vector3(0,boundsY - 1.3f);
            NameText.text = HostClass.Name;
            break;
        case "Names_Common":
            break;
        case "Falling":
            if (!HostClass.Dead){
                float Velo = UnityEngine.Mathf.Abs(HostClass.RigBody.velocity.y);
                if (Velo > 0.2f){ //am in air
                    HostClass.InAir = true;
                    HostClass.SetState("InAir");
                }else{
                    HostClass.InAir = false;
                    if (HostClass.AniState == "InAir"){
                        HostClass.SetState("Stand");
                    }
                }
            }
            break;
        case "Damage_FromEnvo":
            break;
        case "Damage_FromShot":
            break;
        case "Damage_FriendlyFire":
            break;
        case "Slap_Meme":
            if (!HostClass.Dead){
                if(HostClass.AniState == "Stand"){
                    GameObject slapobj = HostClass.gameObject;

                    RaycastHit2D slapHit = Physics2D.Raycast(
                        slapobj.transform.position + new Vector3(0,0.5f),
                        slapobj.transform.right * slapobj.transform.localScale.x,
                        2,
                        1 << LayerMask.NameToLayer("Objects"),
                        -UnityEngine.Mathf.Infinity,
                        UnityEngine.Mathf.Infinity);

                    if (slapHit) {
                        GameObject thinghit = slapHit.collider.gameObject;
                        if (thinghit.GetComponent<Rigidbody2D>() != null){
                            Rigidbody2D hitrigi = thinghit.GetComponent<Rigidbody2D>();
                            if (hitrigi.bodyType != RigidbodyType2D.Static){
                                hitrigi.velocity = new Vector2((slapobj.transform.right.x * slapobj.transform.localScale.x) * UnityEngine.Random.Range(30, 150), UnityEngine.Random.Range(0, 40));
                                NHCfunc.PlaySFX(AssetLoadup.SFX_Attack_Slap, slapobj.transform.position);
                            }
                        }
                    }
                }
            }
            break;
        case "Talking":
            SpriteRenderer MyTalkComp = TalkingBubble.GetComponent<SpriteRenderer>();

            if (UnityEngine.Time.fixedTime > TalkStateTimer){ //talking state change delay and code.
                TalkStateTimer = (UnityEngine.Time.fixedTime + TalkStateAmt);
                TalkStateAmt = UnityEngine.Random.Range(1f, 7f);
                Talking = !Talking;
                if (Talking){
                    if (HostClass.AniState == "Stand" && AIchancePoolCheck()){
                    HostClass.SetState("Talking");
                    MyTalkComp.enabled = true;
                    TalkChangeTimer = (UnityEngine.Time.fixedTime + 1); //tricks the frame to change, workaround O_O
                    }else{
                        Talking = false;
                        MyTalkComp.enabled = false;
                        if (HostClass.AniState == "Talking"){
                        HostClass.SetState("Stand");
                        }
                    }
                }else{
                    MyTalkComp.enabled = false;
                    if (HostClass.AniState == "Talking"){
                    HostClass.SetState("Stand");
                    }
                }
            }

            if (!HostClass.Dead){
                if ((!(HostClass.AniState == "Stand" || HostClass.AniState == "Talking")) && Talking){
                    Talking = false;
                    MyTalkComp.enabled = false;
                }
                
                if (Talking){ //talking tick code and also frame variation code
                    if (UnityEngine.Time.fixedTime > TalkChangeTimer){
                        TalkChangeTimer = (UnityEngine.Time.fixedTime + TalkChangeAmt);
                        TalkChangeAmt = UnityEngine.Random.Range(0.8f, 4f);
                        MyTalkComp.sprite = AssetLoadup.Talk_Bubbles[UnityEngine.Random.Range(0, (AssetLoadup.Talk_Bubbles.Length - 1))];
                    }

                        TalkingBubble.transform.localScale = HostClass.gameObject.transform.localScale;
                        boundsY = ((HostClass.gameObject.GetComponent<Collider2D>().bounds.size.y / HostClass.gameObject.transform.localScale.y));
                        TalkingBubble.transform.localPosition = new Vector3(0,boundsY - 1f);
                }
            }else{
                if (Talking) {
                    Talking = false;
                    MyTalkComp.enabled = false;
                }
            }
            break;
        case "Jumping":
            if(HostClass.PlayerControlling){
                if (InputSystem.Held(AssetLoadup.KeyUp)){
                    HostClass.Jump(true);
                }
            }
            break;
        case "JumpGaps":
            if (HostClass.AniState == "Walk"){
                if (!(HostClass.RigBody.velocity.y > 2.5f)){
                    float FeetBounds = ((HostClass.gameObject.GetComponent<Collider2D>().bounds.size.y * 0.75f) + 0.3f);
                    RaycastHit2D HitObj = Physics2D.Raycast(HostClass.gameObject.transform.position, -HostClass.gameObject.transform.up, FeetBounds, 1 << LayerMask.NameToLayer("Objects"), -UnityEngine.Mathf.Infinity, UnityEngine.Mathf.Infinity);
                    RaycastHit2D HitBounds = Physics2D.Raycast(HostClass.gameObject.transform.position, -HostClass.gameObject.transform.up, FeetBounds, 1 << LayerMask.NameToLayer("Bounds"), -UnityEngine.Mathf.Infinity, UnityEngine.Mathf.Infinity);
                    float GapHitGoal = float.PositiveInfinity;

                    if (HitBounds){ GapHitGoal = HitBounds.distance; } //gets what is the shortest distance aka the closest hit
                    if (HitObj) {if (GapHitGoal > HitObj.distance) GapHitGoal = HitObj.distance;}
                    if (GapHitGoal > (FeetBounds - 0.3f)){
                        HostClass.Jump();
                    }
                }
            }
            break;
        default:
            ModAPI.Notify("<color=#ff35005>NHC AI module " + Name + " func not found</color>");
            break;
        }

    }

    public bool AIchancePoolCheck(){ //used to compare an even chance of This Thing being executed- via a bool, compared to other AI module actions that normally happen while standing or etc
        return (((int) UnityEngine.Mathf.Round(UnityEngine.Random.Range(0, HostClass.AIchancePool))) == 0);
    }

    public int CompareTo(AImodule other){
        if(other == null)return 1;
        return 0;
    }
}
// Originally uploaded by 'Abyssal Studio'. Do not reupload without their explicit permission.
