using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using System.Linq;

public struct AssetLoadup
{
    public static Canvas Canvas;

    public static float PixPerUnit;
    public static Sprite ControlSprite;

    public static Sprite[] Char_Human_Stand;
    public static Sprite[] Char_Human_Walk;
    public static Sprite[] Char_Human_Attack;
    public static Sprite[] Char_Human_Death;
    public static Sprite[] Char_Human_Gun;
    public static Sprite[] Char_Human_InAir;
    public static Sprite[] Char_Human_Talk;

    public static Sprite[] Char_Human_Stand_Hurt;
    public static Sprite[] Char_Human_Walk_Hurt;
    public static Sprite[] Char_Human_Attack_Hurt;
    public static Sprite[] Char_Human_Gun_Hurt;
    public static Sprite[] Char_Human_InAir_Hurt;
    public static Sprite[] Char_Human_Talk_Hurt;

    public static Sprite[] Talk_Bubbles;

    public static Sprite UI_Basic;
    public static Sprite UI_Basic_Alt;
    public static Sprite UI_Window;
    public static Sprite UI_ExitButton;
    public static Sprite UI_ExitButton_Alt;

    public static Sprite[] HealthIcons;

    public static AudioClip[] SFX_Average_Death;
    public static AudioClip[] SFX_Average_Default;
    public static AudioClip[] SFX_Cyber_Death;
    public static AudioClip[] SFX_Cyber_Default;
    public static AudioClip[] SFX_Large_Death;
    public static AudioClip[] SFX_Large_Default;
    public static AudioClip[] SFX_Undead_Death;
    public static AudioClip[] SFX_Undead_Default;
    public static AudioClip[] SFX_Attack_Default_Miss;
    public static AudioClip[] SFX_Attack_Default;
    public static AudioClip[] SFX_Attack_Pistol;
    public static AudioClip[] SFX_Attack_Slap;
    public static AudioClip[] SFX_Attack_Undead;
    public static AudioClip[] SFX_Body_HitGround;
    public static AudioClip[] SFX_Gore;
    public static AudioClip[] SFX_Scary;
    public static AudioClip[] SFX_Step_Default;
    public static AudioClip[] SFX_Step_Large;
    public static AudioClip[] SFX_Jump;
    
    public static AudioClip[] SFX_Average_Talk;

    public static AudioClip[] SFX_Revive;

    public static string KeyLeft;
    public static string KeyRight;
    public static string KeyUp;
    public static string KeyMain;
    public static string KeyAlt;

    public static void Init(){ //sets stuff up
        InitCharFrames();
        InitSFX();
        InitDefinitions();
        InitUI();
    }

    public static void InitDefinitions()
    {
        KeyLeft = "NHC Controller Left";
        KeyRight ="NHC Controller Right";
        KeyUp = "NHC Controller Up";
        KeyMain = "NHC Controller ActMain";
        KeyAlt = "NHC Controller ActAlt";
    }

    public static void InitCharFrames(){ //all nhc character sprites
        ControlSprite = ModAPI.LoadSprite("chars/char_human_col.png"); //needed for internal usage, the exact sprite doesn't matter, just the pixels per unit
        PixPerUnit = ControlSprite.pixelsPerUnit;

        Char_Human_Stand = NHCfunc.LoadCharFrames("char_human_stand",1);
        Char_Human_Walk = NHCfunc.LoadCharFrames("char_human_walk",4);
        Char_Human_Attack = NHCfunc.LoadCharFrames("char_human_attack",3);
        Char_Human_Death = NHCfunc.LoadCharFrames("char_human_death",4);
        Char_Human_Gun = NHCfunc.LoadCharFrames("char_human_gun",4);
        Char_Human_InAir = NHCfunc.LoadCharFrames("char_human_inair",1);
        Char_Human_Talk = NHCfunc.LoadCharFrames("char_human_talk",1);

        Char_Human_Stand_Hurt = NHCfunc.LoadCharFrames("char_human_stand",1,true);
        Char_Human_Walk_Hurt = NHCfunc.LoadCharFrames("char_human_walk",4,true);
        Char_Human_Attack_Hurt = NHCfunc.LoadCharFrames("char_human_attack",3,true);
        Char_Human_Gun_Hurt = NHCfunc.LoadCharFrames("char_human_gun",4,true);
        Char_Human_InAir_Hurt = NHCfunc.LoadCharFrames("char_human_inair",1,true);
        Char_Human_Talk_Hurt = NHCfunc.LoadCharFrames("char_human_talk",1,true);

        Talk_Bubbles = NHCfunc.LoadCharFrames("talk/",39);

        HealthIcons = NHCfunc.LoadCharFrames("health/",11);
    }

    public static void InitSFX(){ //all nhc sfx audioclips
        SFX_Average_Death = NHCfunc.LoadSFX("ambi_average_death",3);
        SFX_Average_Default = NHCfunc.LoadSFX("ambi_average_default",2);
        SFX_Cyber_Death = NHCfunc.LoadSFX("ambi_cyber_death",1);
        SFX_Cyber_Default = NHCfunc.LoadSFX("ambi_cyber_default",1);
        SFX_Large_Death = NHCfunc.LoadSFX("ambi_large_death",1);
        SFX_Large_Default = NHCfunc.LoadSFX("ambi_large_default",2);
        SFX_Undead_Death = NHCfunc.LoadSFX("ambi_undead_death",1);
        SFX_Undead_Default = NHCfunc.LoadSFX("ambi_undead_default",1);
        SFX_Attack_Default_Miss = NHCfunc.LoadSFX("attack_default_miss",1);
        SFX_Attack_Default = NHCfunc.LoadSFX("attack_default",1);
        SFX_Attack_Pistol = NHCfunc.LoadSFX("attack_pistol",1);
        SFX_Attack_Slap = NHCfunc.LoadSFX("slap",1);
        SFX_Attack_Undead = NHCfunc.LoadSFX("attack_undead",3);
        SFX_Body_HitGround = NHCfunc.LoadSFX("body_hit_ground",1);
        SFX_Gore = NHCfunc.LoadSFX("gore",1);
        SFX_Scary = NHCfunc.LoadSFX("scary",1);
        SFX_Step_Default = NHCfunc.LoadSFX("step_default",1);
        SFX_Step_Large = NHCfunc.LoadSFX("step_large",1);
        SFX_Jump = NHCfunc.LoadSFX("jump",1);

        SFX_Average_Talk = NHCfunc.LoadSFX("talk/average_talk",14);

        SFX_Revive = NHCfunc.LoadSFX("revive",1);

    }

    public static void InitUI(){
        UI_Basic = NHCfunc.LoadUISprite("basic_0", new Vector4(2,2,2,2));
        UI_Basic_Alt = NHCfunc.LoadUISprite("basic_1", new Vector4(2,2,2,2));
        UI_Window = NHCfunc.LoadUISprite("window", new Vector4(15,15,15,15));
        UI_ExitButton = NHCfunc.LoadUISprite("button_exit_0", new Vector4(2,2,2,2));
        UI_ExitButton_Alt = NHCfunc.LoadUISprite("button_exit_1", new Vector4(2,2,2,2));

    }

    public static string[] CommonNames(){
        string[] TempNames = new string[] {
            "James",
            "Mary",
            "Robert",
            "Rob",
            "Patricia",
            "John",
            "Jennifer",
            "Jenny",
            "Michael",
            "Mick",
            "Mike",
            "Linda",
            "William",
            "Willy",
            "Elizabeth",
            "Eliza",
            "Liz",
            "David",
            "Dave",
            "Barbara",
            "Richard",
            "Richy",
            "Susan",
            "Sus",
            "Joseph",
            "Joe",
            "Jessica",
            "Jess",
            "Thomas",
            "Tom",
            "Sarah",
            "Charles",
            "Karen",
            "Christopher",
            "Chris",
            "Nancy",
            "Daniel",
            "Dan",
            "Lisa",
            "Matthew",
            "Mat",
            "Betty",
            "Anthony",
            "Margaret",
            "Mark",
            "Sandra",
            "Sandy",
            "Donald",
            "Don",
            "Ashley",
            "Ash",
            "Steven",
            "Steve",
            "Kimberly",
            "Kim",
            "Paul",
            "Emily",
            "Emm",
            "Andrew",
            "Andy",
            "Donna",
            "Joshua",
            "Josh",
            "Michelle",
            "Kenneth",
            "Dorothy",
            "Kevin",
            "Carol",
            "Brian",
            "Amanda",
            "George",
            "Melissa",
            "Edward",
            "Deborah",
            "Ronald",
            "Stephanie",
            "Timothy",
            "Jason",
            "Gary",
            "Gaz",
            "Amy",
            "Eric",
            "Jonathan",
            "Angela",
            "Larry",
            "Scott",
            "Brandon",
            "Gregory",
            "Frank",
            "Freddy",
            "Jack",
            "Rachel",
            "Jerry",
            "Adam",
            "Peter",
            "Olivia",
            "Olive"
        };
        return(TempNames);
    }
}

// Originally uploaded by 'Abyssal Studio'. Do not reupload without their explicit permission.
